<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

/**
 * Heler for managing the apps 
 * @since 2.2
 */
class PayplansHelperAppmanager
{
	const SUCCESS 	= 200;
	const ERROR		= 500;
	const NOT_FOUND = 404;

	/**
	 * contains the list of apps from remote file
	 * @var Array
	 * @since 2.2
	 */
	public static $applist = array();
	
	/**
	 * load the app list from remote file
	 * @since 2.2
	 */
	protected static function loadList()
	{
		$url 	  = trim(self::getServerUrl(), '/').'&object=content&action=list';	
		$contents = PayplansHelperUtils::postDataByCurl($url, 'version='.self::getPayplansVersion());	
		if ($contents == false){
			return false;
		}
	
		self::$applist = json_decode($contents);	

		self::cacheFile('appville.json',$contents);
	}
	
	/**
	 * returns the list of all available apps
	 * @since 2.2
	 * @return Array
	 */
	public function getAppList()
	{
		$pathToJsonFile = JPATH_ROOT.DS."media".DS."payplans".DS."app".DS."appmanager".DS;
		$appFilename	= "appville.json";
		
		if(!file_exists($pathToJsonFile.DS.$appFilename)){
			self::loadList();
		}
		else{
			$contents		= file_get_contents($pathToJsonFile.DS.$appFilename);
			return json_decode($contents);
		}
		
		if(self::$applist === array()){
			self::loadList();
		}
		
		return self::$applist;
	}
	
	/**
	 * marks the app installed or not in the applist
	 * @since 2.2
	 * @return array(stdclass)
	 * $requireExt created from appville.json this are the apps that we need to fetch not the all
	 */
	public static function getInstalledPlugins()
	{
		static $plugins = array();
		
		if(count($plugins)){
			return $plugins;
		}
		
		$ext_mapping = array('plugin'=>'plg','module'=>'mod','component'=>'com','library'=>'lib','language'=>'lang','file'=>'file');
		
		$query = new XiQuery();
		
		$extensions = $query->select('*')->from('`#__extensions`')->dbLoadQuery()->loadObjectList();

		foreach ($extensions as $extension)
		{
			$extension->xml_params = json_decode($extension->manifest_cache);
			$version = (isset($extension->xml_params->version))?array_pop(explode('.', $extension->xml_params->version)):0;
			$extension->build_version = $version;
			$extension->type = (isset($ext_mapping[$extension->type]))? $ext_mapping[$extension->type] : 'plg';
			$plugins[$extension->type.'_'.$extension->folder.'_'.$extension->element.'_'.$extension->client_id] = $extension;
		}
		
		return $plugins;
	}
	
	/**
	 * returns the release apps withe therie build number
	 * @since 2.2
	 * @return Array 
	 */
	public static function getReleasedAppVersion()
	{
		
		$pathToJsonFile = JPATH_ROOT.DS."media".DS."payplans".DS."app".DS."appmanager".DS;
		$relesFilename 		= "release.json";
		
		if(!file_exists($pathToJsonFile.DS.$relesFilename)){
			$url	 = self::getServerUrl().'&object=release&action=list';
			$content = PayplansHelperUtils::postDataByCurl($url, 'version='.self::getPayplansVersion());		
			
			self::cacheFile($relesFilename,$content);
			$content = json_decode($content);
			return $content;
		}
		else{
			$content		= file_get_contents($pathToJsonFile.DS.$relesFilename);
			return json_decode($content);
		}
		
		// XITODO : do all these stuff on cron		
	}
	
	/**
	 * un-install the extension
	 * @param integer $eid
	 * @since 2.2
	 * @return boolean
	 */
	public static function uninstall($eid)
	{
		require_once JPATH_ADMINISTRATOR.DS.'components'.DS.'com_installer'.DS.'models'.DS.'manage.php';		
		$model = new InstallerModelManage();
		return $model->remove(array($eid));
	} 
	
	/**
	 * install the extension
	 * @param $app_element string
	 * @param $app_folder string
	 * @since 2.2 
	 */
	public static function install($app_folder, $app_element, $extension_type, $client_id)
	{
		list($user_name, $password) = self::getCredential();
		
		$version = self::getPayplansVersion();
		
		$string 			= "app_element=".$app_element
							."&app_folder=".$app_folder
							."&username=".$user_name
							."&password=".$password
							."&version=".$version
							."&extension_type=".$extension_type
							."&client_id=".$client_id;
							
		$url 				= self::getServerUrl().'&object=release&action=fetch';
				
		//XITODO : disable the debug mode before asking for file
		list($info, $response)	= PayplansHelperUtils::postDataByCurl($url, $string, true);
		
		
		if(strtolower($info) === 'application/zip'){
			$tmp_file_name 	 	= JPATH_ROOT.DS.'tmp'.DS.$extension_type.'_'.$app_folder.'_'.$app_element.'_'.$client_id.'.zip';
			$tmp_folder_name 	= JPATH_ROOT.DS.'tmp'.DS.$extension_type.'_'.$app_folder.'_'.$app_element.'_'.$client_id;
			
			// create a file
			JFile::write($tmp_file_name, $response);		
			
			jimport('joomla.filesystem.archive');
			jimport( 'joomla.installer.installer' );
			jimport('joomla.installer.helper');
			JArchive::extract($tmp_file_name, $tmp_folder_name);
			$installer = JInstaller::getInstance();	
			
			if($installer->install($tmp_folder_name)){
				if($extension_type == 'plg'){
					XiHelperPlugin::changeState($app_element, $app_folder,'1');
				}			$response = json_encode(array('response_code' => self::SUCCESS, 'error_code' => 'INSTALLATION_SUCCESS'));
			}
			else{
				$response = json_encode(array('response_code' => self::ERROR, 'error_code' => 'INSTALLATION_ERROR'));
			}
			
			JFolder::delete($tmp_folder_name);
			JFile::delete($tmp_file_name);
		}
			
		return $response;
	}
	
	/**
	 * return the user name password  
	 */
	public static function getCredential()
	{
		$config 	= PayplansFactory::getConfig();
		$username 	= isset($config->jpayplansUsername) ? $config->jpayplansUsername : '';
		$password 	= isset($config->jpayplansPassword) ? $config->jpayplansPassword : ''; 
		return array($username, $password);
	}
	
	/**
	 * set the username & password in configuration  
	 */
	public static function setCredential($username, $password)
	{	
		$url 	= self::getServerUrl().'&object=user&action=verify';
		$string = 'username='.$username.'&password='.$password;

		list($info, $response)	= PayplansHelperUtils::postDataByCurl($url, $string, true);
			
		$decoded_response = json_decode($response);
		if($decoded_response->response_code == self::ERROR){
			return $response;
		}
		
		if(!PayplansConfig::getInstance(PAYPLANS_CONFIG_ADVANCE)
							->bind(array('config'=>array('jpayplansUsername' => $username, 'jpayplansPassword' => $password)))
							->save())
							{
								$response['response_code'] 	= self::ERROR;
								$response['error_code']		= 'ERROR_IN_SAVE';
								return json_encode($response);
							};	
							
		return $response;
	}
	
	public static function getServerUrl()
	{
		static $server_url = null;
		
		if($server_url === null){
			$file_path  = 'http://pub.jpayplans.com/app-manager/server.json';						
			$contents   =  json_decode(self::makeCurlRequestToReadFile($file_path));
			$server_url = $contents->server_url.'/index.php?option=com_payplans&plugin=ppappserver';
		}
		
		return $server_url;
	}
	
	public function getPayplansVersion()
	{
		$version = explode(".", PAYPLANS_VERSION);
		unset($version[2]);
		return implode(".", $version);
	}
	
	/*---function to create an curl request to read file content from server---*/
	public function makeCurlRequestToReadFile($file_path)
	{
			$curl 	    = curl_init();
			curl_setopt ($curl, CURLOPT_URL, $file_path);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
			$response   = curl_exec($curl);
			curl_close($curl);
			
			return $response;
	}
	
	public function cacheFile($filename,$content){
		
		$folderPath = JPATH_ROOT.DS."media".DS."payplans".DS."app".DS."appmanager";
		if(!is_dir($folderPath)){
			mkdir($folderPath);
		}

		$handler 		= fopen($folderPath.DS.$filename,"w+");
		fwrite($handler, $content);
		fclose($handler);
	}
	
	public function fetchInstalledExtensions($app_folder, $app_element, $extension_type, $client_id)
	{
		switch ($extension_type)
		{
			case 'plg' : $extension_type = 'plugin';
							 break;
							 
			case 'mod' : $extension_type = 'module';
							 break;
			
			case 'com' : $extension_type = 'component';
				 			 break;
				
			case 'lang' : $extension_type = 'language';
							  break;
			
			case 'lib' : $extension_type = 'library';
							  break;
							 
			case 'file'	    : $extension_type = 'file';
							  break;
							
			default		: $extension_type = 'plugin';
							 
		}
		
		$db 	= PayplansFactory::getDbo();
		$query 	= "select * from `#__extensions` where `type` ='$extension_type' and `element` = '$app_element' and `folder`= '$app_folder' and `client_id`= '$client_id'";		
		$db->setQuery($query);
		$extension = $db->loadObject();
		
		return $extension->extension_id;
	}

}

