<?php
/**
 * iMaQma Helpdesk Component
 * www.imaqma.com
 *
 * @package iMaQma Helpdesk
 * @copyright (C) 2000-2011 Components Lab, Lda.
 * @license GNU/GPL
 * 
 * $Id: admin.priority.php 337 2012-01-22 17:24:30Z pdaniel $
 * $LastChangedDate: 2012-01-22 17:24:30 +0000 (dom, 22 Jan 2012) $
 *
 */

/** ensure this file is being included by a parent file */
defined( '_JEXEC' ) or die( 'Direct Access to this location is not allowed.' );

require_once( "components/com_maqmahelpdesk/html/admin.$function.html.php" );

// get parameters from the URL or submitted form
$cid = JRequest::getVar( 'cid', array(0), '', 'array' );
if (!is_array( $cid )) {
	$cid = array(0);
}

// Activities logger
mqm_activitylog('admin', 'priority', $task, $cid[0]);

switch ($task) {
	case "new":
		editPriority( 0 );
		break;
	case "edit":
		editPriority( $cid[0] );
		break;
	case "save":
		savePriority();
		break;
	case "remove":
		removePriority( $cid );
		break;
	case "publish":
		publishPriority( $cid, 1 );
		break;
	case "unpublish":
		publishPriority( $cid, 0 );
		break;
	default:
		showPriority();
		break;
}

/**
* Compiles a list of priorities
* @param string The component name
*/
function showPriority() {
	$database = JFactory::getDBO();
	$mainframe = JFactory::getApplication();
	$limit = $mainframe->getUserStateFromRequest( "global.list.limit", 'limit', $mainframe->getCfg('list_limit'), 'int');
	$limitstart = intval( JRequest::getVar( 'limitstart', 0, '', 'int' ) );

	// get the total number of records
	$database->setQuery( "SELECT count(*) FROM #__support_priority ORDER BY description" );
	$total = $database->loadResult();
	echo $database->getErrorMsg();

	jimport('joomla.html.pagination');;
	$pageNav = new JPagination( $total, $limitstart, $limit );

	$database->setQuery( "SELECT p.id, p.description, p.show, p.timeunit, p.timevalue, p.isdefault"
	. "\nFROM #__support_priority p"
	. "\nORDER BY p.description",
	$pageNav->limitstart, $pageNav->limit
	);

	$rows = $database->loadObjectList();
	if ($database->getErrorNum()) {
		echo $database->stderr();
		return false;
	}

	priority_html::show( $rows, $pageNav );
}

/**
* Compiles information to add or edit a priority
* @param database A database connector object
* @param string The component name
*/
function editPriority( $uid=0 ) {
	$database = JFactory::getDBO();
	$row = new mosSupportPriority( $database );
	// load the row from the db table
	$row->load( $uid );

	$timelist[] = JHTML::_('select.option', '0', JText::_('selectlist'));
	$timelist[] = JHTML::_('select.option', 'H', JText::_('hours'));
	$timelist[] = JHTML::_('select.option', 'D', JText::_('days'));
	$lists['timeunit'] = JHTML::_('select.genericlist',$timelist, 'timeunit', 'class="inputbox" size="1"', 'value', 'text', $row->timeunit);
	
	$captions = array( JText::_('MQ_NO'), JText::_('MQ_YES') );
	$values = array( '0', '1' );
	
	$lists['default'] = mqm_switchcheckbox('radio', 'isdefault', $captions, $values, $row->isdefault, 'switch');
	$lists['show'] = mqm_switchcheckbox('radio', 'show', $captions, $values, $row->show, 'switch');
	
	priority_html::edit( $row, $lists );
}

/**
* Saves the workgroup
* @param database A database connector object
* @param string The name of the component
*/
function savePriority() {
	$database = JFactory::getDBO();
	$mainframe = JFactory::getApplication();
	$row = new mosSupportPriority( $database );
	
	if( $_POST['isdefault'] ) {
		$database->setQuery( "UPDATE #__support_priority SET isdefault='0'" );
		$database->query();
	}

	if (!$row->bind( $_POST )) {
		echo "<script type='text/javascript'> alert('".$row->getErrorMsg()."'); window.history.go(-1); </script>\n";
		exit();
	}
	if (!$row->check()) {
		echo "<script type='text/javascript'> alert('".$row->getErrorMsg()."'); window.history.go(-1); </script>\n";
		exit();
	}
	if (!$row->store()) {
		echo "<script type='text/javascript'> alert('".$row->getErrorMsg()."'); window.history.go(-1); </script>\n";
		exit();
	}

	$row->checkin();

	$mainframe->redirect( "index.php?option=com_maqmahelpdesk&task=priority" );
}

/**
* Deletes one or more categories from the categories table
* @param database A database connector object
* @param string The name of the priority section
* @param array An array of unique priority id numbers
*/
function removePriority( $cid ) {
	$database = JFactory::getDBO();
	$mainframe = JFactory::getApplication();
	
	if (count( $cid ) < 1) {
		echo "<script type='text/javascript'> alert('".JText::_('priority_delete')."'); window.history.go(-1);</script>\n";
		exit;
	}

	if (count( $cid )) {
		$cids = implode( ',', $cid );
		$database->setQuery( "DELETE FROM #__support_priority WHERE id IN (".$database->quote($cids).")" );
		if (!$database->query()) {
			echo "<script type='text/javascript'> alert('".$database->getErrorMsg()."'); window.history.go(-1); </script>\n";
		}
	}

	$mainframe->redirect( "index.php?option=com_maqmahelpdesk&task=priority" );
}

/**
* Publishes or Unpublishes one or more workgroups
* @param database A database connector object
* @param array An array of unique workgroup id numbers
* @param integer 0 if unpublishing, 1 if publishing
* @param string The name of the component
*/
function publishPriority( $cid=null, $publish=1 ) {
	$database = JFactory::getDBO();
	$mainframe = JFactory::getApplication();

	if ( !is_array( $cid ) || count( $cid ) < 1 ) {
		$action = $publish ? 'publish' : 'unpublish';
		echo "<script type='text/javascript'> alert('".JText::_('priority_action')." ".$action."'); window.history.go(-1);</script>\n";
		exit;
	}

	$cids = implode( ',', $cid );
	$count = count( $cid );

	$database->setQuery( "UPDATE #__support_priority SET `show`='$publish' WHERE id IN (".$database->quote($cids).")" );

	if (!$database->query()) {
		echo "<script type='text/javascript'> alert('".$database->getErrorMsg()."'); window.history.go(-1); </script>\n";
		exit();
	}

	$mainframe->redirect( "index.php?option=com_maqmahelpdesk&task=priority" );
}
?>