<?php
/**
* @copyright	Copyright (C) 2009 - 2009 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	COMMUNITY ACL
* @contact 		shyam@readybytes.in
* 
*/
if(defined('_JEXEC')===false) die();

class PayplansAppCommunity_acl extends PayplansApp
{
	protected $_location	= __FILE__;
	protected $_resource	= 'com_community_acl.group';
	
	public function _isApplicable(PayplansIfaceApptriggerable $refObject)
	{
		return JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_community_acl');
	}

	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}
		
		$newStatus  = $new->getStatus();
		
		$active = $this->getAppParam('addToGroupOnActive');
		$active	= (is_array($active)) ? $active : array($active);
		
		$hold 	= $this->getAppParam('addToGroupOnHold');
		$hold	= (is_array($hold)) ? $hold : array($hold);
		
		$expire = $this->getAppParam('addToGroupOnExpire');
		$expire	= (is_array($expire)) ? $expire : array($expire);
			
		$subid  = $new->getId();
		$userid = $new->getBuyer();
		
		// if subscription is active
		if($newStatus == XiStatus::SUBSCRIPTION_ACTIVE){
			$this->_removeFromGroup($userid, $hold, $subid);
			$this->_removeFromGroup($userid, $expire, $subid);
			$this->_addToGroup($userid, $active, $subid);
			
			return true;
		}
		
		//if subscription is hold
		if($newStatus == XiStatus::SUBSCRIPTION_HOLD){
			$this->_removeFromGroup($userid, $active, $subid);
			$this->_removeFromGroup($userid, $expire, $subid);
			$this->_addToGroup($userid, $hold, $subid);
			
			return true;
		}
		// if subscription is active
		if($newStatus == XiStatus::SUBSCRIPTION_EXPIRED){
			$this->_removeFromGroup($userid, $active, $subid);
			$this->_removeFromGroup($userid, $hold, $subid);
			$this->_addToGroup($userid, $expire, $subid);
			
			return true;
		}
		
		return true;
	}
	
	protected function _addToGroup( $userid, $groupids, $subid )
	{
		if(!is_array($groupids)){
			return false;
		}		

		foreach($groupids as $groupid){
			if(empty($groupid)){
				continue;
			}
			
			$users = $this->_getGroupMembers($groupid);
			
			if(in_array($userid, $users)){
				$this->_addToResource($subid, $userid, $groupid, $this->_resource);
				continue;
			}
			
			
			if($this->_setGroupMember($groupid, $userid)){
				$this->_addToResource($subid, $userid, $groupid, $this->_resource);
				$message = sprintf(XiText::_('COM_PAYPLANS_LOGGER_COMMUNITY_ACL_LOG_ADDED_TO_GROUP'), $userid, $groupid);
				$content = array( 'added_user'=>$userid , 'community_acl_user_group' => $groupid);
				PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $this, $content);
			}
		}
		
		return true;
	}
	
	protected function _removeFromGroup( $userid, $groupids, $subid )
	{
		if(!is_array($groupids)){
			return false;
		}
		
		foreach($groupids as $groupid){
			if(empty($groupid)){
				continue;
			}
			
			$users = $this->_getGroupMembers($groupid);
			
			if(!in_array($userid,$users)){
				continue;
			}
					
			if($this->_removeFromResource($subid, $userid, $groupid, $this->_resource) 
					&& $this->_removeGroupMember($groupid, $userid)){
				$message = sprintf(XiText::_('COM_PAYPLANS_LOGGER_COMMUNITY_ACL_LOG_REMOVED_FROM_GROUP'), $userid, $groupid);
				$content = array( 'removed_user'=>$userid , 'community_acl_user_group' => $groupid);
				PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $this, $content);
			}
		}
		
		return true;
	}
	
	protected function _getGroupMembers($groupid)
	{
		$db = JFactory::getDBO();

		$query = 'SELECT `user_id`'
			. ' FROM #__community_acl_users'
			. ' WHERE `group_id` = \'' . $groupid . '\''
			;
		$db->setQuery( $query );
		$users = $db->loadResultArray();
		return $users;
	}
	
	protected function _setGroupMember($groupid, $userid)
	{		
		$db = JFactory::getDBO();
		
		$query = 'INSERT INTO `#__community_acl_users`'
			. ' (`user_id`, `group_id`)'
			. ' VALUES ( "' . $userid . '", "' . $groupid. '")';

		$db->setQuery( $query );
		return $db->query();
	}
	
	protected function _removeGroupMember($groupid, $userid)
	{	
		$db = JFactory::getDBO();
		
		$query = 'DELETE FROM `#__community_acl_users` '
				.' WHERE `user_id` = "' . $userid.'" '
				.' AND `group_id` = "' . $groupid.'"';
		
		$db->setQuery( $query );
		return $db->query();
	}
	
	//render Widget
	public function renderWidgetHtml()
	{   
		//get user id
		$userid     = XiFactory::getUser()->id;
		if(!$userid){
			return '';
		}
		//get user's community ACL groups
	    $community_acl_groups = $this->_getUserCommunityACLGroup($userid);
	    if(empty($community_acl_groups)){
	    	return '';
	    }
	    $this->assign('community_acl_groups',$community_acl_groups);
        $data = $this->_render('widgethtml');
        return $data;
	}	
	
	//get Community ACL user group
	protected function _getUserCommunityACLGroup($userid)
	{
		$db         	      = JFactory::getDBO();
		$community_acl_groups = array();
		$query      	      = 'SELECT groups.name '
			              . ' FROM #__community_acl_users as users, #__community_acl_groups as groups' 
			              . ' WHERE `user_id` = ' . $userid .' AND groups.id = users.group_id ';
		$db->setQuery( $query );	
        $community_acl_groups  = $db->loadResultArray();
        return $community_acl_groups;
	}
	
	function getNameFromResourceValue($resource, $value)
	{
		// if its a different resource
		if($resource != $this->_resource){
			return false;
		}
		$db         = XiFactory::getDBO();		
		$query      = 'SELECT `name`'
			          . ' FROM #__community_acl_groups'
			          . ' WHERE `id` = "'. $value.'"';
		$db->setQuery( $query );
		return $db->loadResult();	
	
	}
}
